/*******
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
K_KeyHandling created by Kakashi24142 on September 19, 2008
///////////////////////////////////////////////////////////

K_KeyHandling is a library that allows for dynamic macros to be created. You can add and remove macros at runtime and
allow for procs to be associated with macros and not only verbs. Also, the delay to detect a "hold" event is set to .2
seconds so that will be very useful for fast-paced games. This library can also do a few more things; read on to find
out!

The first and foremost thing you must do in order to have it set up is override the initializeMacros() proc. This proc
initializes the macros list so that you can have macros ready.

client/initializeMacros()
	macros=new/Macros(mob,
					  list(Macro(mob,"A",
					  			 Command("PressProc"),
					  			 Command("HoldProc"),
					  			 Command("ReleaseProc")
	                            )
	                      )
	                 )


I'll explain each part of the code. It's actually a one-line piece of code, but I split it up into different lines for
the sake of readability. Basically, you are initializing the macros list which contains all the macros that client has
access to.

Macros/New(mob/ref,list/macros)

Those are the arguments I created for initializing the Macros datum. The first is the mob who owns the verb/proc you
want triggered after a certain key has been pressed. (Usually that's just the client's mob). The next argument is a
list datum containing all the macros a player has. (Note that this list CAN be changed at runtime, I will explain
later.)

Macro/New(mob/ref,key,Command/press,Command/hold,Command/release)

Those are the arguments I created for initializing the Macro datum. The first is the mob who owns the verb/proc you
want  triggered after a certain key has been pressed. The second is the name of the key in which you want to attach a
proc/verb to. Examples are "A","B","4","NORTH","SOUTH", etc. (Enter them just as you would see them in the macro
editor.) The rest of the arguments are the Command datums that store information on what commands to trigger in each
of the events (a press, a hold, and a release).
*Note: press,hold, and release arguments can be null, so it is not required for every macro to have all those
commands.

Command/New(procname,list/args)

Those are the arguments I created for initializing the Command datum. The first is the name of the proc or verb you
want (in a text string) and the second is a list or arguments you want to pass to it.
*Note: procname is a mandatory argument, but the args argument can be left null.

!!!!!!
EXTRAS
!!!!!!

This is a bit of extra information you might want to read if you plan on customization.

global var:
	macroid="macro"
		This var is used to name the id of the macro element in the interface file. If there is none, then by default
		it is macro.

++++++++++++++
Macros (datum)
++++++++++++++

vars:
	Macro/list/macros   //list containing all the Macro datums
	mob/ref	            //reference to the mob that acts upon key events

procs:
	Retrieve(key)
		This proc takes one argument which is the key for which a macro is added. This proc returns a reference to a
		Macro datum in case you ever need to modify variables in a Macro. You can pass as many arguments as you want.
		Example:
			var/Macro/M=client.macros.Retrieve("A")  //Retrieve the Macro datum with "A" as its key.
			M.hold_delay=5                           //Change the delay at which the "hold" event occurs to 5
			M.press=Command("Press")                 //Change the press command to a proc/verb called "Press"

	Add(Macro/macro1,Macro/macro2,...)
		This proc is used to add new Macros into the Macros datum.
		Example:
			client.macros.Add(Macro(src,"B",Command("Press"))) //Adds a macro that calls upon the Press proc when the
			                                                   //B key is pressed.

	Remove(key1,key2,...)
		This proc is used to remove exisiting Macros in the Macros datum. You can pass as many arguments as you want.
		Example:
			client.macros.Remove("A","B","C","D")

	Refresh()
		Resets the macros in the interface file so that the macros in the list can be used. This proc rarely ever
		needs to be used explicitly.

+++++++++++++
Macro (datum)
+++++++++++++

vars:
	*The first four were previously explained.
	key
	Command/press
	Command/hold
	Command/release
	hold_delay=2
		This is the delay before the "hold" event is trigger. The default delay is .2 seconds and it is the minumum.
		Reducing the value of this variable less then 2 will screw things up so be careful.
	tmp
		*The following tmp variables should NOT be meddled with, or else things will get messed up.
		press_time=0 //The time of the press
		status=0     //The status can be either RELEASED(0), PRESS(1), HOLD(2).
		mob/ref      //The mob who the macro shall be triggered for.

proc:
	Trigger(Command/command,mob/who)
		This proc uses the call proc to call the proc specified in the Command datum in the argument with the source
		begin the who mob. This proc rarely ever needs to be used explicitly.
+++++++++++++++
Command (datum)
+++++++++++++++

vars:
	list/args     //list of arguments to send
	procname      //name of the proc/verb

If you want to use this library further, take a look at Implementation.dm. It should be fairly easy to understand how
it works.

------------------------------------------------------------
Know of any bugs? Suggestions? Questions? Please contact me:
-IM me on MSN at kakashi24142@hotmail.com
-IM me on AIM at kakashi24142
-Email me at kakashisensei95@gmail.com
------------------------------------------------------------
********/