//EventLoop library
#include <deadron/eventloop>
//some constants
#define RELEASED  0
#define PRESSED   1
#define HELD      2

var/macroid="macro"

//The following two are just meant as shortcuts
proc/Macro()
	return new/Macro(arglist(args))

proc/Command()
	return new/Command(arglist(args))

client
	var/Macros/Macros

	New()

		.=..()

		initializeMacros()

	proc
		initializeMacros()   //override this proc such that it will cotain the default macros.

	verb
		KeyPressed(key as text)
			set name=".kp"                         //short for Key Pressed

			var/Macro/M=Macros.Retrieve(key)

			if(M && (M.status==RELEASED))

				M.status=PRESSED
				M.press_time=world.time
				M.Trigger(M.press,mob)

		KeyReleased(key as text)
			set name=".kr"                         //short for Key Released

			var/Macro/M=Macros.Retrieve(key)

			if(M)

				M.status=RELEASED
				M.press_time=0
				M.Trigger(M.release,mob)

Macros
	var
		Macro/list/macros
		tmp
			mob/ref

	New(mob/ref,macros[])

		src.ref=ref

		if(macros && macros.len)

			for(var/Macro/M in macros)

				Add(M)

	proc

		Retrieve(key)

			if(istext(key) && macros && macros.len)

				for(var/Macro/M in macros)

					if(M.key==key)

						return M

		Add()

			if(!macros) macros=new

			for(var/Macro/M in args)

				var/Macro/Check=Retrieve(M.key)

				if(Check) macros.Remove(Check)

				macros.Add(M)

			Refresh()

		Remove()

			if(macros && macros.len)

				for(var/key in args)

					var/Macro/M=Retrieve(key)

					if(M) macros.Remove(M)

				if(!macros.len) macros=null

				Refresh()


		Refresh()

			for(var/Macro/M in macros)
			    //Generates the macro entry for key press
				winset(ref,"[M.key]",{"parent=[macroid];name=[M.key];command='.kp "[M.key]"'"})
				//Generates the macro entry for key release
				winset(ref,"[M.key]+UP",{"parent=[macroid];name=[M.key]+UP;command='.kr "[M.key]"'"})


Macro
	var
		key
		Command/press
		Command/hold
		Command/release
		hold_delay=2 //tenth of a second
		tmp
			press_time=0
			status=0
			mob/ref

	New(mob/ref,key,Command/press,Command/hold,Command/release)

		ASSERT(ismob(ref) && key)

		src.ref=ref
		src.key=key

		src.press=press
		src.hold=hold
		src.release=release

		GameController.AddEventCycleReceiver(src)

	base_EventCycle(world_time)
		if((status==PRESSED) && (world_time >= (press_time+hold_delay)-1))

			status=HELD

		else if(status==HELD)

			Trigger(hold,ref)

	proc
		Trigger(Command/command,mob/who)

			if(istype(command) && istype(who) && hascall(who,command.procname))

				call(who,command.procname)(arglist(command.args))

Command
	var
		list/args     //list of arguments to send
		procname      //name of the proc/verb

	New(procname,list/args)

		ASSERT(istext(procname) && (isnull(args) || istype(args)))

		src.procname=procname
		src.args=args



